@@ -1,32 +1,41 @@
 /*
-    SPDX-FileCopyrightText: 2010 Tobias Koenig <tokoe@kde.org>
+    Copyright (c) 2010 Tobias Koenig <tokoe@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-or-later
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "davitemslistjob.h"
-#include "davjobbase_p.h"
 
 #include "daverror.h"
-#include "davmanager_p.h"
-#include "davprotocolbase_p.h"
+#include "davmanager.h"
+#include "davprotocolbase.h"
 #include "davurl.h"
+#include "utils.h"
 #include "etagcache.h"
-#include "utils_p.h"
 
 #include <KIO/DavJob>
 #include <KIO/Job>
 
-#include <set>
+#include <QtCore/QBuffer>
+#include <QtCore/QDebug>
 
 using namespace KDAV;
 
-namespace KDAV
-{
-class DavItemsListJobPrivate : public DavJobBasePrivate
-{
+class DavItemsListJobPrivate {
 public:
-    void davJobFinished(KJob *job);
+    DavItemsListJobPrivate(const DavUrl &url, std::shared_ptr<EtagCache> cache);
 
     DavUrl mUrl;
     std::shared_ptr<EtagCache> mEtagCache;
@@ -34,44 +43,49 @@ public:
     QString mRangeStart;
     QString mRangeEnd;
     DavItem::List mItems;
-    std::set<QString> mSeenUrls; // to prevent events duplication with some servers
+    QSet<QString> mSeenUrls; // to prevent events duplication with some servers
     DavItem::List mChangedItems;
     QStringList mDeletedItems;
-    uint mSubJobCount = 0;
+    uint mSubJobCount;
 };
-}
 
-DavItemsListJob::DavItemsListJob(const DavUrl &url, const std::shared_ptr<EtagCache> &cache, QObject *parent)
-    : DavJobBase(new DavItemsListJobPrivate, parent)
+DavItemsListJobPrivate::DavItemsListJobPrivate(const DavUrl &url, std::shared_ptr<EtagCache> cache)
+    : mUrl(url)
+    , mEtagCache(cache)
+    , mSubJobCount(0)
 {
-    Q_D(DavItemsListJob);
-    d->mUrl = url;
-    d->mEtagCache = cache;
 }
 
-DavItemsListJob::~DavItemsListJob() = default;
+
+DavItemsListJob::DavItemsListJob(const DavUrl &url, std::shared_ptr<EtagCache> cache, QObject *parent)
+    : DavJobBase(parent)
+    , d(std::unique_ptr<DavItemsListJobPrivate>(new DavItemsListJobPrivate(url, cache)))
+{
+}
+
+DavItemsListJob::~DavItemsListJob()
+{
+}
 
 void DavItemsListJob::setContentMimeTypes(const QStringList &types)
 {
-    Q_D(DavItemsListJob);
     d->mMimeTypes = types;
 }
 
 void DavItemsListJob::setTimeRange(const QString &start, const QString &end)
 {
-    Q_D(DavItemsListJob);
     d->mRangeStart = start;
     d->mRangeEnd = end;
 }
 
 void DavItemsListJob::start()
 {
-    Q_D(DavItemsListJob);
-    const DavProtocolBase *protocol = DavManager::davProtocol(d->mUrl.protocol());
+    const DavProtocolBase *protocol = DavManager::self()->davProtocol(d->mUrl.protocol());
     Q_ASSERT(protocol);
+    QVectorIterator<XMLQueryBuilder::Ptr> it(protocol->itemsQueries());
 
-    const auto queries = protocol->itemsQueries();
-    for (XMLQueryBuilder::Ptr builder : queries) {
+    while (it.hasNext()) {
+        XMLQueryBuilder::Ptr builder = it.next();
         if (!d->mRangeStart.isEmpty()) {
             builder->setParameter(QStringLiteral("start"), d->mRangeStart);
         }
@@ -85,56 +99,49 @@ void DavItemsListJob::start()
         if (d->mMimeTypes.isEmpty() || d->mMimeTypes.contains(mimeType)) {
             ++d->mSubJobCount;
             if (protocol->useReport()) {
-                KIO::DavJob *job = DavManager::self()->createReportJob(d->mUrl.url(), props.toString());
+                KIO::DavJob *job = DavManager::self()->createReportJob(d->mUrl.url(), props);
                 job->addMetaData(QStringLiteral("PropagateHttpHeader"), QStringLiteral("true"));
                 job->setProperty("davType", QStringLiteral("report"));
                 job->setProperty("itemsMimeType", mimeType);
-                connect(job, &KIO::DavJob::result, this, [d](KJob *job) {
-                    d->davJobFinished(job);
-                });
+                connect(job, &KIO::DavJob::result, this, &DavItemsListJob::davJobFinished);
             } else {
-                KIO::DavJob *job = DavManager::self()->createPropFindJob(d->mUrl.url(), props.toString());
+                KIO::DavJob *job = DavManager::self()->createPropFindJob(d->mUrl.url(), props);
                 job->addMetaData(QStringLiteral("PropagateHttpHeader"), QStringLiteral("true"));
                 job->setProperty("davType", QStringLiteral("propFind"));
                 job->setProperty("itemsMimeType", mimeType);
-                connect(job, &KIO::DavJob::result, this, [d](KJob *job) {
-                    d->davJobFinished(job);
-                });
+                connect(job, &KIO::DavJob::result, this, &DavItemsListJob::davJobFinished);
             }
         }
     }
 
     if (d->mSubJobCount == 0) {
         setError(ERR_ITEMLIST_NOMIMETYPE);
-        d->setErrorTextFromDavError();
+        setErrorTextFromDavError();
         emitResult();
     }
 }
 
 DavItem::List DavItemsListJob::items() const
 {
-    Q_D(const DavItemsListJob);
     return d->mItems;
 }
 
 DavItem::List DavItemsListJob::changedItems() const
 {
-    Q_D(const DavItemsListJob);
     return d->mChangedItems;
 }
 
 QStringList DavItemsListJob::deletedItems() const
 {
-    Q_D(const DavItemsListJob);
     return d->mDeletedItems;
 }
 
-void DavItemsListJobPrivate::davJobFinished(KJob *job)
+void DavItemsListJob::davJobFinished(KJob *job)
 {
     KIO::DavJob *davJob = qobject_cast<KIO::DavJob *>(job);
-    const int responseCode = davJob->queryMetaData(QStringLiteral("responsecode")).isEmpty() //
-        ? 0
-        : davJob->queryMetaData(QStringLiteral("responsecode")).toInt();
+    const int responseCode = davJob->queryMetaData(QStringLiteral("responsecode")).isEmpty() ?
+                             0 :
+                             davJob->queryMetaData(QStringLiteral("responsecode")).toInt();
 
     // KIO::DavJob does not set error() even if the HTTP status code is a 4xx or a 5xx
     if (davJob->error() || (responseCode >= 400 && responseCode < 600)) {
@@ -170,12 +177,12 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
          */
 
         const QString itemsMimeType = job->property("itemsMimeType").toString();
-        QDomDocument document;
-        document.setContent(davJob->responseData(), QDomDocument::ParseOption::UseNamespaceProcessing);
+        const QDomDocument document = davJob->response();
         const QDomElement documentElement = document.documentElement();
 
         QDomElement responseElement = Utils::firstChildElementNS(documentElement, QStringLiteral("DAV:"), QStringLiteral("response"));
         while (!responseElement.isNull()) {
+
             QDomElement propstatElement;
 
             // check for the valid propstat, without giving up on first error
@@ -184,7 +191,7 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
                 for (int i = 0; i < propstats.length(); ++i) {
                     const QDomElement propstatCandidate = propstats.item(i).toElement();
                     const QDomElement statusElement = Utils::firstChildElementNS(propstatCandidate, QStringLiteral("DAV:"), QStringLiteral("status"));
-                    if (statusElement.text().contains(QLatin1String("200"))) {
+                    if (statusElement.text().contains(QStringLiteral("200"))) {
                         propstatElement = propstatCandidate;
                     }
                 }
@@ -197,9 +204,9 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
 
             const QDomElement propElement = Utils::firstChildElementNS(propstatElement, QStringLiteral("DAV:"), QStringLiteral("prop"));
 
-            // check whether it is a DAV collection ...
+            // check whether it is a dav collection ...
             const QDomElement resourcetypeElement = Utils::firstChildElementNS(propElement, QStringLiteral("DAV:"), QStringLiteral("resourcetype"));
-            if (!resourcetypeElement.isNull()) {
+            if (!responseElement.isNull()) {
                 const QDomElement collectionElement = Utils::firstChildElementNS(resourcetypeElement, QStringLiteral("DAV:"), QStringLiteral("collection"));
                 if (!collectionElement.isNull()) {
                     responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
@@ -225,45 +232,38 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
                 url = QUrl::fromUserInput(href);
             }
 
-            const QString itemUrl = url.toDisplayString();
-            const auto [it, isInserted] = mSeenUrls.insert(itemUrl);
-            if (!isInserted) {
+            QString itemUrl = url.toDisplayString();
+            if (d->mSeenUrls.contains(itemUrl)) {
                 responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
                 continue;
             }
 
+            d->mSeenUrls << itemUrl;
             auto _url = url;
-            _url.setUserInfo(mUrl.url().userInfo());
-            item.setUrl(DavUrl(_url, mUrl.protocol()));
+            _url.setUserInfo(d->mUrl.url().userInfo());
+            item.setUrl(DavUrl(_url, d->mUrl.protocol()));
 
-            // extract ETag
+            // extract etag
             const QDomElement getetagElement = Utils::firstChildElementNS(propElement, QStringLiteral("DAV:"), QStringLiteral("getetag"));
 
             item.setEtag(getetagElement.text());
 
-            mItems << item;
+            d->mItems << item;
 
-            if (mEtagCache->etagChanged(itemUrl, item.etag())) {
-                mChangedItems << item;
+            if (d->mEtagCache->etagChanged(itemUrl, item.etag())) {
+                d->mChangedItems << item;
             }
 
             responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
         }
     }
 
-    mDeletedItems.clear();
+    QSet<QString> removed = d->mEtagCache->urls().toSet();
+    removed.subtract(d->mSeenUrls);
+    d->mDeletedItems = removed.toList();
 
-    const auto map = mEtagCache->etagCacheMap();
-    for (auto it = map.cbegin(); it != map.cend(); ++it) {
-        const QString remoteId = it.key();
-        if (mSeenUrls.find(remoteId) == mSeenUrls.cend()) {
-            mDeletedItems.append(remoteId);
-        }
-    }
-
-    if (--mSubJobCount == 0) {
+    if (--d->mSubJobCount == 0) {
         emitResult();
     }
 }
 
-#include "moc_davitemslistjob.cpp"
